@@ -24,9 +24,10 @@ from boto.cloudformation.stack import Output
 from moto.cloudformation.models import FakeStack, CloudFormationBackend, cloudformation_backends
 from moto.cloudformation.exceptions import ValidationError, UnformattedGetAttTemplateException
 from localstack import config
-from localstack.constants import DEFAULT_PORT_CLOUDFORMATION_BACKEND, TEST_AWS_ACCOUNT_ID, MOTO_ACCOUNT_ID
+from localstack.constants import TEST_AWS_ACCOUNT_ID, MOTO_ACCOUNT_ID
 from localstack.utils.aws import aws_stack, aws_responses
-from localstack.utils.common import FuncThread, short_uid, recurse_object, clone, json_safe, md5, canonical_json
+from localstack.utils.common import (
+    FuncThread, short_uid, recurse_object, clone, json_safe, md5, canonical_json, get_free_tcp_port)
 from localstack.stepfunctions import models as sfn_models
 from localstack.services.infra import (
     get_service_protocol, start_proxy_for_service, do_run, canonicalize_api_names
@@ -37,6 +38,8 @@ from localstack.services.cloudformation import service_models
 
 LOG = logging.getLogger(__name__)
 
+MOTO_CFN_ACCOUNT_ID = '123456789'
+
 # Maps (stack_name,resource_logical_id) -> Bool to indicate which resources are currently being updated
 CURRENTLY_UPDATING_RESOURCES = {}
 
@@ -69,9 +72,9 @@ MODEL_MAP = {
 
 def start_cloudformation(port=None, asynchronous=False, update_listener=None):
     port = port or config.PORT_CLOUDFORMATION
-    backend_port = DEFAULT_PORT_CLOUDFORMATION_BACKEND
     print('Starting mock CloudFormation service in %s ports %s (recommended) and %s (deprecated)...' % (
         get_service_protocol(), config.EDGE_PORT, port))
+    backend_port = get_free_tcp_port()
     start_proxy_for_service('cloudformation', port, backend_port, update_listener)
     if RUN_SERVER_IN_PROCESS:
         cmd = 'python "%s" cloudformation -p %s -H 0.0.0.0' % (__file__, backend_port)
@@ -169,10 +172,24 @@ def update_physical_resource_id(resource):
         elif isinstance(resource, service_models.ElasticsearchDomain):
             resource.physical_resource_id = resource.params.get('DomainName')
 
+        elif isinstance(resource, dynamodb_models.Table):
+            resource.physical_resource_id = resource.name
+
+        elif isinstance(resource, dynamodb2_models.Table):
+            resource.physical_resource_id = resource.name
+
         else:
             LOG.warning('Unable to determine physical_resource_id for resource %s' % type(resource))
 
 
+def update_resource_name(resource, resource_json):
+    """ Some resources require minor fixes in their CF resource definition
+        before we can pass them on to deployment. """
+    props = resource_json['Properties'] = resource_json.get('Properties') or {}
+    if isinstance(resource, sfn_models.StateMachine) and not props.get('StateMachineName'):
+        props['StateMachineName'] = resource.name
+
+
 def apply_patches():
     """ Apply patches to make LocalStack seamlessly interact with the moto backend.
         TODO: Eventually, these patches should be contributed to the upstream repo! """
@@ -180,6 +197,9 @@ def apply_patches():
     # add model mappings to moto
     parsing.MODEL_MAP.update(MODEL_MAP)
 
+    # fix account ID
+    parsing.ACCOUNT_ID = TEST_AWS_ACCOUNT_ID
+
     # Patch clean_json in moto
     def clean_json(resource_json, resources_map):
         result = clean_json_orig(resource_json, resources_map)
@@ -230,14 +250,18 @@ def apply_patches():
         resource_tuple = parsing.parse_resource(logical_id, resource_json, resources_map)
         if not resource_tuple:
             return None
-        _, resource_json, _ = resource_tuple
+        _, resource_json, resource_name = resource_tuple
 
-        def add_default_props(resource_props):
+        def add_default_props(resource_props, res_name=None):
             """ apply some fixes which otherwise cause deployments to fail """
             res_type = resource_props['Type']
             props = resource_props.get('Properties', {})
             if res_type == 'AWS::Lambda::EventSourceMapping' and not props.get('StartingPosition'):
                 props['StartingPosition'] = 'LATEST'
+
+            if res_type == 'AWS::ApiGateway::RestApi':
+                props['Name'] = props.get('Name') or res_name
+
             # generate default names for certain resource types
             default_attrs = (('AWS::IAM::Role', 'RoleName'), ('AWS::Events::Rule', 'Name'))
             for entry in default_attrs:
@@ -245,7 +269,7 @@ def apply_patches():
                     props[entry[1]] = 'cf-%s-%s' % (stack_name, md5(canonical_json(props)))
 
         # add some fixes and default props which otherwise cause deployments to fail
-        add_default_props(resource_json)
+        add_default_props(resource_json, resource_name)
         for resource in resources_map._resource_json_map.values():
             add_default_props(resource)
 
@@ -267,6 +291,9 @@ def apply_patches():
                 resource = parse_and_create_resource_orig(
                     logical_id, resource_json_arns_fixed, resources_map, region_name
                 )
+                if not resource:
+                    # this can happen if the resource has an associated Condition which evaluates to false
+                    return resource
                 resource.logical_id = logical_id
             except Exception as e:
                 moto_create_error = e
@@ -353,13 +380,6 @@ def apply_patches():
 
         return resource
 
-    def update_resource_name(resource, resource_json):
-        """ Some resources require minor fixes in their CF resource definition
-            before we can pass them on to deployment. """
-        props = resource_json['Properties'] = resource_json.get('Properties') or {}
-        if isinstance(resource, sfn_models.StateMachine) and not props.get('StateMachineName'):
-            props['StateMachineName'] = resource.name
-
     def update_resource_id(resource, new_id, props, region_name, stack_name, resource_map):
         """ Update and fix the ID(s) of the given resource. """
 
@@ -588,6 +608,20 @@ def apply_patches():
     SNS_Topic_get_cfn_attribute_orig = sns_models.Topic.get_cfn_attribute
     sns_models.Topic.get_cfn_attribute = SNS_Topic_get_cfn_attribute
 
+    # Patch create_from_cloudformation_json(..)
+    # #2568 Cloudformation create-stack for SNS with yaml causes TypeError
+
+    SNS_Topic_create_from_cloudformation_json_orig = sns_models.Topic.create_from_cloudformation_json
+
+    def SNS_Topic_create_from_cloudformation_json(resource_name, cloudformation_json, region_name):
+        properties = cloudformation_json['Properties']
+        if properties.get('Subscription'):
+            properties['Subscription'] = [subscription for subscription in properties['Subscription'] if subscription]
+
+        return SNS_Topic_create_from_cloudformation_json_orig(resource_name, cloudformation_json, region_name)
+
+    sns_models.Topic.create_from_cloudformation_json = SNS_Topic_create_from_cloudformation_json
+
     # Patch ES get_cfn_attribute(..) method
     def ES_get_cfn_attribute(self, attribute_name):
         if attribute_name in ['Arn', 'DomainArn']:
@@ -626,7 +660,13 @@ def apply_patches():
         func_name = props.get('FunctionName') or ''
         if ':lambda:' in func_name:
             props['FunctionName'] = aws_stack.lambda_function_name(func_name)
-        return Mapping_create_from_cloudformation_json_orig(resource_name, cloudformation_json, region_name)
+        try:
+            return Mapping_create_from_cloudformation_json_orig(resource_name, cloudformation_json, region_name)
+        except Exception:
+            LOG.info('Unable to add Lambda event mapping for source ARN "%s" in moto backend (ignoring)' %
+                props.get('EventSourceArn'))
+            # return an empty dummy instance, to avoid downstream None value issues
+            return service_models.BaseModel()
 
     Mapping_create_from_cloudformation_json_orig = lambda_models.EventSourceMapping.create_from_cloudformation_json
     lambda_models.EventSourceMapping.create_from_cloudformation_json = Mapping_create_from_cloudformation_json
@@ -673,7 +713,7 @@ def apply_patches():
     @classmethod
     def RestAPI_create_from_cloudformation_json(cls, resource_name, cloudformation_json, region_name):
         props = cloudformation_json['Properties']
-        name = props['Name']
+        name = props.get('Name') or short_uid()
         region_name = props.get('Region') or aws_stack.get_region()
         description = props.get('Description') or ''
         id = props.get('Id') or short_uid()
@@ -811,8 +851,9 @@ def apply_patches():
         FuncThread(run_loop).start()
 
     def initialize_resources(self):
+        self.resource_map._template = self.resource_map._template or self.template_dict
         self.resource_map.load()
-        self.resource_map.create()
+        self.resource_map.create(self.template_dict)
         self.output_map.create()
         run_dependencies_deployment_loop(self, 'CREATE')
 
@@ -867,6 +908,35 @@ def apply_patches():
 
     CloudFormationBackend.create_change_set = cloudformation_backend_create_change_set
 
+    # patch cloudformation backend change_set methods
+    # #2240 - S3 bucket not created since 0.10.8
+    # #2568 - Cloudformation create-stack for SNS with yaml causes TypeError
+    cloudformation_backend_describe_change_set_orig = CloudFormationBackend.describe_change_set
+    cloudformation_backend_execute_change_set_orig = CloudFormationBackend.execute_change_set
+
+    def cloudformation_backend_describe_change_set(self, change_set_name, stack_name=None):
+        change_set_name = change_set_name.replace(TEST_AWS_ACCOUNT_ID, MOTO_CFN_ACCOUNT_ID)
+        return cloudformation_backend_describe_change_set_orig(self, change_set_name, stack_name)
+
+    def cloudformation_backend_execute_change_set(self, change_set_name, stack_name=None):
+        change_set_name = change_set_name.replace(TEST_AWS_ACCOUNT_ID, MOTO_CFN_ACCOUNT_ID)
+        resp = cloudformation_backend_execute_change_set_orig(self, change_set_name, stack_name)
+
+        stack = self.change_sets.get(change_set_name)
+        if not stack:
+            for cs in self.change_sets:
+                if self.change_sets[cs].change_set_name == change_set_name:
+                    stack = self.change_sets[cs]
+
+        stack.output_map = stack._create_output_map()
+        for export in stack.exports:
+            self.exports[export.name] = export
+
+        return resp
+
+    CloudFormationBackend.describe_change_set = cloudformation_backend_describe_change_set
+    CloudFormationBackend.execute_change_set = cloudformation_backend_execute_change_set
+
 
 def inject_stats_endpoint():
     """ Inject a simple /_stats endpoint into the moto server backend Web app. """
@@ -878,7 +948,7 @@ def inject_stats_endpoint():
         all_objects = muppy.get_objects()
         result = summary.summarize(all_objects)
         result = result[0:20]
-        summary = '\n'.join([l for l in summary.format_(result)])
+        summary = '\n'.join([line for line in summary.format_(result)])
         result = '%s\n\n%s' % (summary, json.dumps(result))
         return result, 200, {'content-type': 'text/plain'}
 
