@@ -1,4 +1,5 @@
 import ast
+import base64
 import json
 import uuid
 import logging
@@ -12,12 +13,12 @@ from six.moves.urllib import parse as urlparse
 from localstack.config import external_service_url
 from localstack.constants import TEST_AWS_ACCOUNT_ID, MOTO_ACCOUNT_ID
 from localstack.services.awslambda import lambda_api
-from localstack.services.generic_proxy import ProxyListener
 from localstack.utils.analytics import event_publisher
 from localstack.utils.aws import aws_stack
 from localstack.utils.aws.aws_responses import response_regex_replace
 from localstack.utils.aws.dead_letter_queue import sns_error_to_dead_letter_queue
-from localstack.utils.common import timestamp_millis, short_uid, to_str
+from localstack.utils.common import timestamp_millis, short_uid, to_str, to_bytes
+from localstack.utils.persistence import PersistingProxyListener
 
 # set up logger
 LOG = logging.getLogger(__name__)
@@ -32,7 +33,10 @@ SUBSCRIPTION_STATUS = {}
 SNS_TAGS = {}
 
 
-class ProxyListenerSNS(ProxyListener):
+class ProxyListenerSNS(PersistingProxyListener):
+    def api_name(self):
+        return 'sns'
+
     def forward_request(self, method, path, data, headers):
         if method == 'OPTIONS':
             return 200
@@ -44,9 +48,15 @@ class ProxyListenerSNS(ProxyListener):
         except Exception as e:
             return make_error(message=str(e), code=400)
 
-        if method == 'POST' and path == '/':
+        if method == 'POST':
             # parse payload and extract fields
             req_data = urlparse.parse_qs(to_str(data), keep_blank_values=True)
+
+            # parse data from query path
+            if not req_data:
+                parsed_path = urlparse.urlparse(path)
+                req_data = urlparse.parse_qs(parsed_path.query, keep_blank_values=True)
+
             req_action = req_data['Action'][0]
             topic_arn = req_data.get('TargetArn') or req_data.get('TopicArn') or req_data.get('ResourceArn')
 
@@ -127,10 +137,14 @@ class ProxyListenerSNS(ProxyListener):
 
             elif req_action == 'CreateTopic':
                 topic_arn = aws_stack.sns_topic_arn(req_data['Name'][0])
-                self._extract_tags(topic_arn, req_data)
+                tag_resource_success = self._extract_tags(topic_arn, req_data, True)
+                # in case if there is an error it returns an error , other wise it will continue as expected.
+                if not tag_resource_success:
+                    return make_error(code=400, code_string='InvalidParameter',
+                                  message='Topic already exists with different tags')
 
             elif req_action == 'TagResource':
-                self._extract_tags(topic_arn, req_data)
+                self._extract_tags(topic_arn, req_data, False)
                 return make_response(req_action)
 
             elif req_action == 'UntagResource':
@@ -148,14 +162,22 @@ class ProxyListenerSNS(ProxyListener):
         return True
 
     @staticmethod
-    def _extract_tags(topic_arn, req_data):
+    def _extract_tags(topic_arn, req_data, is_create_topic_request):
         tags = []
         req_tags = {k: v for k, v in req_data.items() if k.startswith('Tags.member.')}
         for i in range(int(len(req_tags.keys()) / 2)):
             key = req_tags['Tags.member.' + str(i + 1) + '.Key'][0]
             value = req_tags['Tags.member.' + str(i + 1) + '.Value'][0]
             tags.append({'Key': key, 'Value': value})
+
+            # this means topic already created with empty tags and when we try to create it
+            # again with other tag value then it should fail according to aws documentation.
+            existing_tags = SNS_TAGS.get(topic_arn, None)
+            if is_create_topic_request and existing_tags is not None and existing_tags != tags:
+                return False
+
         do_tag_resource(topic_arn, tags)
+        return True
 
     @staticmethod
     def _reset_account_id(data):
@@ -166,6 +188,11 @@ class ProxyListenerSNS(ProxyListener):
             data, colon_delimiter='%3A', existing=TEST_AWS_ACCOUNT_ID, replace=MOTO_ACCOUNT_ID)
 
     def return_response(self, method, path, data, headers, response):
+        # persist requests to disk
+        super(ProxyListenerSNS, self).return_response(
+            method, path, data, headers, response
+        )
+
         if method == 'POST' and path == '/':
             # convert account IDs in ARNs
             data = aws_stack.fix_account_id_in_arns(data, colon_delimiter='%3A')
@@ -224,11 +251,11 @@ def unsubscribe_sqs_queue(queue_url):
                 subscriptions.remove(subscriber)
 
 
-def publish_message(topic_arn, req_data, subscription_arn=None):
+def publish_message(topic_arn, req_data, subscription_arn=None, skip_checks=False):
     message = req_data['Message'][0]
     sqs_client = aws_stack.connect_to_service('sqs')
 
-    LOG.debug('Publishing message to TopicArn: %s | Message:  %s' % (topic_arn, message))
+    LOG.debug('Publishing message to TopicArn: %s | Message: %s' % (topic_arn, message))
 
     subscriptions = SNS_SUBSCRIPTIONS.get(topic_arn, [])
     for subscriber in list(subscriptions):
@@ -236,7 +263,8 @@ def publish_message(topic_arn, req_data, subscription_arn=None):
             continue
         filter_policy = json.loads(subscriber.get('FilterPolicy') or '{}')
         message_attributes = get_message_attributes(req_data)
-        if not check_filter_policy(filter_policy, message_attributes):
+        if not skip_checks and not check_filter_policy(filter_policy, message_attributes):
+            LOG.info('SNS filter policy %s does not match attributes %s' % (filter_policy, message_attributes))
             continue
 
         if subscriber['Protocol'] == 'sqs':
@@ -265,16 +293,23 @@ def publish_message(topic_arn, req_data, subscription_arn=None):
 
         elif subscriber['Protocol'] == 'lambda':
             try:
+                external_url = external_service_url('sns')
+                unsubscribe_url = '%s/?Action=Unsubscribe&SubscriptionArn=%s' % (external_url,
+                                    subscriber['SubscriptionArn'])
                 response = lambda_api.process_sns_notification(
                     subscriber['Endpoint'],
                     topic_arn,
                     subscriber['SubscriptionArn'],
                     message,
                     message_attributes,
+                    unsubscribe_url,
                     subject=req_data.get('Subject', [None])[0]
                 )
-                if isinstance(response, FlaskResponse):
+                if isinstance(response, Response):
                     response.raise_for_status()
+                elif isinstance(response, FlaskResponse):
+                    if response.status_code >= 400:
+                        raise Exception('Error response (code %s): %s' % (response.status_code, response.data))
             except Exception as exc:
                 LOG.warning('Unable to run Lambda function on SNS message: %s %s' % (exc, traceback.format_exc()))
                 sns_error_to_dead_letter_queue(subscriber['SubscriptionArn'], req_data, str(exc))
@@ -302,6 +337,7 @@ def publish_message(topic_arn, req_data, subscription_arn=None):
                 )
                 response.raise_for_status()
             except Exception as exc:
+                LOG.info('Received error on sending SNS message, putting to DLQ (if configured): %s' % exc)
                 sns_error_to_dead_letter_queue(subscriber['SubscriptionArn'], req_data, str(exc))
         else:
             LOG.warning('Unexpected protocol "%s" for SNS subscription' % subscriber['Protocol'])
@@ -361,7 +397,7 @@ def do_subscribe(topic_arn, endpoint, protocol, subscription_arn, attributes, fi
                         'To confirm the subscription, visit the SubscribeURL included in this message.'],
             'SubscribeURL': ['%s/?Action=ConfirmSubscription&TopicArn=%s&Token=%s' % (external_url, topic_arn, token)]
         }
-        publish_message(topic_arn, confirmation, subscription_arn)
+        publish_message(topic_arn, confirmation, subscription_arn, skip_checks=True)
 
 
 def do_unsubscribe(subscription_arn):
@@ -508,7 +544,7 @@ def create_sqs_message_attributes(subscriber, attributes):
             'DataType': value['Type']
         }
         if value['Type'] == 'Binary':
-            attribute['BinaryValue'] = value['Value']
+            attribute['BinaryValue'] = base64.decodebytes(to_bytes(value['Value']))
         else:
             attribute['StringValue'] = str(value['Value'])
 
