@@ -3,14 +3,13 @@ import re
 import json
 import time
 import boto3
-import base64
 import logging
 import six
-from six.moves.urllib.parse import quote_plus, unquote_plus
 from localstack import config
 from localstack.constants import (
     REGION_LOCAL, LOCALHOST, MOTO_ACCOUNT_ID, ENV_DEV, APPLICATION_AMZ_JSON_1_1,
     APPLICATION_AMZ_JSON_1_0, APPLICATION_X_WWW_FORM_URLENCODED, TEST_AWS_ACCOUNT_ID)
+from localstack.utils.aws import templating
 from localstack.utils.common import (
     run_safe, to_str, is_string, is_string_or_bytes, make_http_request, is_port_open, get_service_protocol)
 from localstack.utils.aws.aws_models import KinesisStream
@@ -201,7 +200,7 @@ def connect_to_resource(service_name, env=None, region_name=None, endpoint_url=N
 
 
 def connect_to_service(service_name, client=True, env=None, region_name=None, endpoint_url=None,
-        config=None, *args, **kwargs):
+        config=None, verify=False, *args, **kwargs):
     """
     Generic method to obtain an AWS service client using boto3, based on environment, region, or custom endpoint_url.
     """
@@ -214,7 +213,6 @@ def connect_to_service(service_name, client=True, env=None, region_name=None, en
         # Cache clients, as this is a relatively expensive operation
         my_session = get_boto3_session()
         method = my_session.client if client else my_session.resource
-        verify = True
         if not endpoint_url:
             if is_local_env(env):
                 endpoint_url = get_local_service_url(service_name)
@@ -225,73 +223,9 @@ def connect_to_service(service_name, client=True, env=None, region_name=None, en
     return BOTO_CLIENTS_CACHE[cache_key]
 
 
-class VelocityInput:
-    """Simple class to mimick the behavior of variable '$input' in AWS API Gateway integration velocity templates.
-    See: http://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-mapping-template-reference.html"""
-
-    def __init__(self, value):
-        self.value = value
-
-    def path(self, path):
-        from jsonpath_rw import parse
-        value = self.value if isinstance(self.value, dict) else json.loads(self.value)
-        jsonpath_expr = parse(path)
-        result = [match.value for match in jsonpath_expr.find(value)]
-        result = result[0] if len(result) == 1 else result
-        return result
-
-    def json(self, path):
-        return json.dumps(self.path(path))
-
-    def __repr__(self):
-        return '$input'
-
-
-class VelocityUtil:
-    """Simple class to mimick the behavior of variable '$util' in AWS API Gateway integration velocity templates.
-    See: http://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-mapping-template-reference.html"""
-
-    def base64Encode(self, s):
-        if not isinstance(s, str):
-            s = json.dumps(s)
-        encoded_str = s.encode(config.DEFAULT_ENCODING)
-        encoded_b64_str = base64.b64encode(encoded_str)
-        return encoded_b64_str.decode(config.DEFAULT_ENCODING)
-
-    def base64Decode(self, s):
-        if not isinstance(s, str):
-            s = json.dumps(s)
-        return base64.b64decode(s)
-
-    def toJson(self, obj):
-        return obj and json.dumps(obj)
-
-    def urlEncode(self, s):
-        return quote_plus(s)
-
-    def urlDecode(self, s):
-        return unquote_plus(s)
-
-    def escapeJavaScript(self, s):
-        return str(s).replace("'", r"\'")
-
-
-def render_velocity_template(template, context, variables={}, as_json=False):
-    import airspeed
-
-    # run a few fixes to properly prepare the template
-    template = re.sub(r'(^|\n)#\s+set(.*)', r'\1#set\2', template, re.MULTILINE)
-
-    t = airspeed.Template(template)
-    var_map = {
-        'input': VelocityInput(context),
-        'util': VelocityUtil()
-    }
-    var_map.update(variables or {})
-    replaced = t.merge(var_map)
-    if as_json:
-        replaced = json.loads(replaced)
-    return replaced
+# TODO remove from here in the future
+def render_velocity_template(*args, **kwargs):
+    return templating.render_velocity_template(*args, **kwargs)
 
 
 def check_valid_region(headers):
@@ -426,6 +360,11 @@ def log_group_arn(group_name, account_id=None, region_name=None):
     return _resource_arn(group_name, pattern, account_id=account_id, region_name=region_name)
 
 
+def events_rule_arn(rule_name, account_id=None, region_name=None):
+    pattern = 'arn:aws:events:%s:%s:rule/%s'
+    return _resource_arn(rule_name, pattern, account_id=account_id, region_name=region_name)
+
+
 def lambda_function_arn(function_name, account_id=None, region_name=None):
     return lambda_function_or_layer_arn('function', function_name, account_id=account_id, region_name=region_name)
 
@@ -483,14 +422,19 @@ def cognito_user_pool_arn(user_pool_id, account_id=None, region_name=None):
     return _resource_arn(user_pool_id, pattern, account_id=account_id, region_name=region_name)
 
 
-def kinesis_stream_arn(stream_name, account_id=None):
-    account_id = get_account_id(account_id)
-    return 'arn:aws:kinesis:%s:%s:stream/%s' % (get_region(), account_id, stream_name)
+def kinesis_stream_arn(stream_name, account_id=None, region_name=None):
+    pattern = 'arn:aws:kinesis:%s:%s:stream/%s'
+    return _resource_arn(stream_name, pattern, account_id=account_id, region_name=region_name)
 
 
-def firehose_stream_arn(stream_name, account_id=None):
-    account_id = get_account_id(account_id)
-    return ('arn:aws:firehose:%s:%s:deliverystream/%s' % (get_region(), account_id, stream_name))
+def firehose_stream_arn(stream_name, account_id=None, region_name=None):
+    pattern = 'arn:aws:firehose:%s:%s:deliverystream/%s'
+    return _resource_arn(stream_name, pattern, account_id=account_id, region_name=region_name)
+
+
+def es_domain_arn(domain_name, account_id=None, region_name=None):
+    pattern = 'arn:aws:es:%s:%s:domain/%s'
+    return _resource_arn(domain_name, pattern, account_id=account_id, region_name=region_name)
 
 
 def s3_bucket_arn(bucket_name, account_id=None):
@@ -550,12 +494,17 @@ def sqs_receive_message(queue_arn):
     return response
 
 
+def firehose_name(firehose_arn):
+    return firehose_arn.split('/')[-1]
+
+
 def mock_aws_request_headers(service='dynamodb', region_name=None):
     ctype = APPLICATION_AMZ_JSON_1_0
     if service == 'kinesis':
         ctype = APPLICATION_AMZ_JSON_1_1
-    elif service == 'sqs':
+    elif service in ['sns', 'sqs']:
         ctype = APPLICATION_X_WWW_FORM_URLENCODED
+
     access_key = get_boto3_credentials().access_key
     region_name = region_name or get_region()
     headers = {
@@ -575,7 +524,8 @@ def dynamodb_get_item_raw(request):
     headers['X-Amz-Target'] = 'DynamoDB_20120810.GetItem'
     new_item = make_http_request(url=config.TEST_DYNAMODB_URL,
         method='POST', data=json.dumps(request), headers=headers)
-    new_item = json.loads(new_item.text)
+    new_item = new_item.text
+    new_item = new_item and json.loads(new_item)
     return new_item
 
 
@@ -807,7 +757,7 @@ def create_kinesis_stream(stream_name, shards=1, env=None, delete=False):
     if delete:
         run_safe(lambda: stream.destroy(), print_error=False)
     stream.create()
-    stream.wait_for()
+    # Note: Returning the stream without awaiting its creation (via wait_for()) to avoid API call timeouts/retries.
     return stream
 
 
