@@ -8,13 +8,10 @@ import logging
 import warnings
 import threading
 import traceback
-import pip as pip_mod
-import shutil
-try:
-    import subprocess32 as subprocess
-except Exception:
-    import subprocess
+import subprocess
 import six
+import shutil
+import pip as pip_mod
 from datetime import datetime
 from localstack import constants, config
 
@@ -62,9 +59,6 @@ MAIN_CONTAINER_NAME = 'localstack_main'
 # the context of the script that starts the Docker container
 ENV_SCRIPT_STARTING_DOCKER = 'LS_SCRIPT_STARTING_DOCKER'
 
-# semaphore for locking access to Popen
-mutex_popen = threading.Semaphore(1)
-
 
 def bootstrap_installation():
     try:
@@ -190,6 +184,11 @@ def get_docker_container_names():
         return []
 
 
+def get_main_container_ip():
+    cmd = "docker inspect -f '{{range .NetworkSettings.Networks}}{{.IPAddress}}{{end}}' %s" % MAIN_CONTAINER_NAME
+    return run(cmd).strip()
+
+
 def setup_logging():
     # determine and set log level
     log_level = logging.DEBUG if is_debug() else logging.INFO
@@ -214,6 +213,7 @@ def setup_logging():
     # disable some logs and warnings
     warnings.filterwarnings('ignore')
     logging.captureWarnings(True)
+    logging.getLogger('asyncio').setLevel(logging.INFO)
     logging.getLogger('boto3').setLevel(logging.INFO)
     logging.getLogger('s3transfer').setLevel(logging.INFO)
     logging.getLogger('docker').setLevel(logging.WARNING)
@@ -298,6 +298,8 @@ class PortMappings(object):
             for i in range(port[1] - port[0] + 1):
                 self.add(port[0] + i, mapped[0] + i)
             return
+        if int(port) <= 0:
+            raise Exception('Unable to add mapping for invalid port: %s' % port)
         for from_range, to_range in self.mappings.items():
             if not self.in_expanded_range(port, from_range):
                 continue
@@ -337,11 +339,26 @@ def get_docker_image_to_start():
     image_name = os.environ.get('IMAGE_NAME')
     if not image_name:
         image_name = constants.DOCKER_IMAGE_NAME
-        if os.environ.get('USE_LIGHT_IMAGE') in constants.TRUE_STRINGS:
-            image_name = constants.DOCKER_IMAGE_NAME_LIGHT
+        if os.environ.get('USE_LIGHT_IMAGE') in constants.FALSE_STRINGS:
+            image_name = constants.DOCKER_IMAGE_NAME_FULL
     return image_name
 
 
+def extract_port_flags(user_flags, port_mappings):
+    regex = r'-p\s+([0-9]+)(\-([0-9]+))?:([0-9]+)(\-([0-9]+))?'
+    matches = re.match('.*%s' % regex, user_flags)
+    start = end = 0
+    if matches:
+        for match in re.findall(regex, user_flags):
+            start = int(match[0])
+            end = int(match[2] or match[0])
+            start_target = int(match[3] or start)
+            end_target = int(match[5] or end)
+            port_mappings.add([start, end], [start_target, end_target])
+        user_flags = re.sub(regex, r'', user_flags)
+    return user_flags
+
+
 def start_infra_in_docker():
 
     container_name = MAIN_CONTAINER_NAME
@@ -373,15 +390,13 @@ def start_infra_in_docker():
     port_mappings = PortMappings()
 
     # get port ranges defined via DOCKER_FLAGS (if any)
-    regex = r'.*-p\s+([0-9]+)(\-([0-9]+))?:([0-9]+)(\-[0-9]+)?.*'
-    match = re.match(regex, user_flags)
-    start = end = 0
-    if match:
-        start = int(match.group(1))
-        end = int(match.group(3) or match.group(1))
-        port_mappings.add([start, end])
+    user_flags = extract_port_flags(user_flags, port_mappings)
+    plugin_run_params = extract_port_flags(plugin_run_params, port_mappings)
 
     # construct default port mappings
+    if service_ports.get('edge') == 0:
+        service_ports.pop('edge')
+    service_ports.pop('dashboard', None)
     for port in service_ports.values():
         port_mappings.add(port)
 
@@ -505,14 +520,6 @@ class FuncThread(threading.Thread):
 def run(cmd, print_error=True, asynchronous=False, stdin=False,
         stderr=subprocess.STDOUT, outfile=None, env_vars=None, inherit_cwd=False,
         inherit_env=True, tty=False):
-    # don't use subprocess module in Python 2 as it is not thread-safe
-    # http://stackoverflow.com/questions/21194380/is-subprocess-popen-not-thread-safe
-    # TODO: should be removed, now that Python 2 has reached its EOL
-    if six.PY2:
-        import subprocess32 as subprocess
-    else:
-        import subprocess
-
     env_dict = os.environ.copy() if inherit_env else {}
     if env_vars:
         env_dict.update(env_vars)
@@ -531,41 +538,39 @@ def run(cmd, print_error=True, asynchronous=False, stdin=False,
             output = subprocess.check_output(cmd, shell=True, stderr=stderr, env=env_dict, cwd=cwd)
             return output.decode(config.DEFAULT_ENCODING)
 
-        # subprocess.Popen is not thread-safe, hence use a mutex here.. (TODO: mutex still needed?)
-        with mutex_popen:
-            stdin_arg = subprocess.PIPE if stdin else None
-            stdout_arg = open(outfile, 'ab') if isinstance(outfile, six.string_types) else outfile
-            stderr_arg = stderr
-            if tty:
-                # Note: leave the "pty" import here (not supported in Windows)
-                import pty
-                master_fd, slave_fd = pty.openpty()
-                stdin_arg = slave_fd
-                stdout_arg = stderr_arg = None
+        stdin_arg = subprocess.PIPE if stdin else None
+        stdout_arg = open(outfile, 'ab') if isinstance(outfile, six.string_types) else outfile
+        stderr_arg = stderr
+        if tty:
+            # Note: leave the "pty" import here (not supported in Windows)
+            import pty
+            master_fd, slave_fd = pty.openpty()
+            stdin_arg = slave_fd
+            stdout_arg = stderr_arg = None
 
-            # start the actual sub process
-            kwargs = {}
-            if is_linux() or is_mac_os():
-                kwargs['preexec_fn'] = os.setsid
-            process = subprocess.Popen(cmd, shell=True, stdin=stdin_arg, bufsize=-1,
-                stderr=stderr_arg, stdout=stdout_arg, env=env_dict, cwd=cwd, **kwargs)
+        # start the actual sub process
+        kwargs = {}
+        if is_linux() or is_mac_os():
+            kwargs['preexec_fn'] = os.setsid
+        process = subprocess.Popen(cmd, shell=True, stdin=stdin_arg, bufsize=-1,
+            stderr=stderr_arg, stdout=stdout_arg, env=env_dict, cwd=cwd, **kwargs)
 
-            if tty:
-                # based on: https://stackoverflow.com/questions/41542960
-                def pipe_streams(*args):
-                    while process.poll() is None:
-                        r, w, e = select.select([sys.stdin, master_fd], [], [])
-                        if sys.stdin in r:
-                            d = os.read(sys.stdin.fileno(), 10240)
-                            os.write(master_fd, d)
-                        elif master_fd in r:
-                            o = os.read(master_fd, 10240)
-                            if o:
-                                os.write(sys.stdout.fileno(), o)
+        if tty:
+            # based on: https://stackoverflow.com/questions/41542960
+            def pipe_streams(*args):
+                while process.poll() is None:
+                    r, w, e = select.select([sys.stdin, master_fd], [], [])
+                    if sys.stdin in r:
+                        d = os.read(sys.stdin.fileno(), 10240)
+                        os.write(master_fd, d)
+                    elif master_fd in r:
+                        o = os.read(master_fd, 10240)
+                        if o:
+                            os.write(sys.stdout.fileno(), o)
 
-                FuncThread(pipe_streams).start()
+            FuncThread(pipe_streams).start()
 
-            return process
+        return process
     except subprocess.CalledProcessError as e:
         if print_error:
             print("ERROR: '%s': exit code %s; output: %s" % (cmd, e.returncode, e.output))
