@@ -10,16 +10,19 @@ import importlib
 from six import iteritems
 from localstack.utils.aws import aws_stack
 from localstack.constants import (
-    LOCALSTACK_ROOT_FOLDER, LOCALSTACK_VENV_FOLDER, LAMBDA_TEST_ROLE, TEST_AWS_ACCOUNT_ID
-)
+    LOCALSTACK_ROOT_FOLDER, LOCALSTACK_VENV_FOLDER, LAMBDA_TEST_ROLE, TEST_AWS_ACCOUNT_ID, ENV_INTERNAL_TEST_RUN)
 from localstack.utils.common import TMP_FILES, run, mkdir, to_str, load_file, save_file, is_alpine
 from localstack.services.awslambda.lambda_api import (
-    get_handler_file_from_name, LAMBDA_DEFAULT_HANDLER, LAMBDA_DEFAULT_RUNTIME, LAMBDA_DEFAULT_STARTING_POSITION,
-    LAMBDA_DEFAULT_TIMEOUT
-)
+    get_handler_file_from_name, LAMBDA_DEFAULT_HANDLER, LAMBDA_DEFAULT_RUNTIME, LAMBDA_DEFAULT_STARTING_POSITION)
 
 ARCHIVE_DIR_PREFIX = 'lambda.archive.'
 DEFAULT_GET_LOG_EVENTS_DELAY = 3
+LAMBDA_TIMEOUT_SEC = 6
+
+
+def is_local_test_mode():
+    """ Whether we are running in the context of our local integration tests. """
+    return bool(os.environ.get(ENV_INTERNAL_TEST_RUN))
 
 
 def copy_dir(source, target):
@@ -127,7 +130,7 @@ def create_zip_file(file_path, get_content=False):
 
 
 def create_lambda_function(func_name, zip_file=None, event_source_arn=None, handler_file=None,
-        handler=LAMBDA_DEFAULT_HANDLER, starting_position=None, runtime=None, envvars={},
+        handler=None, starting_position=None, runtime=None, envvars={},
         tags={}, libs=[], delete=False, layers=None, **kwargs):
     """Utility method to create a new function via the Lambda API"""
 
@@ -137,8 +140,13 @@ def create_lambda_function(func_name, zip_file=None, event_source_arn=None, hand
 
     # load zip file content if handler_file is specified
     if not zip_file and handler_file:
-        zip_file = create_lambda_archive(load_file(handler_file), libs=libs,
-            get_content=True, runtime=runtime or LAMBDA_DEFAULT_RUNTIME)
+        if libs or not handler:
+            zip_file = create_lambda_archive(load_file(handler_file), libs=libs,
+                get_content=True, runtime=runtime or LAMBDA_DEFAULT_RUNTIME)
+        else:
+            zip_file = create_zip_file(handler_file, get_content=True)
+
+    handler = handler or LAMBDA_DEFAULT_HANDLER
 
     if delete:
         try:
@@ -157,7 +165,7 @@ def create_lambda_function(func_name, zip_file=None, event_source_arn=None, hand
         'Code': {
             'ZipFile': zip_file
         },
-        'Timeout': LAMBDA_DEFAULT_TIMEOUT,
+        'Timeout': LAMBDA_TIMEOUT_SEC,
         'Environment': dict(Variables=envvars),
         'Tags': tags
     }
@@ -329,19 +337,27 @@ def get_lambda_log_events(function_name, delay_time=DEFAULT_GET_LOG_EVENTS_DELAY
         time.sleep(delay_time)
 
         logs = aws_stack.connect_to_service('logs')
-        rs = logs.filter_log_events(
-            logGroupName=get_lambda_log_group_name(function_name)
-        )
+        log_group_name = get_lambda_log_group_name(function_name)
+        rs = logs.filter_log_events(logGroupName=log_group_name)
 
         return rs['events']
 
-    events = get_log_events(function_name, delay_time)
+    try:
+        events = get_log_events(function_name, delay_time)
+    except Exception as e:
+        if 'ResourceNotFoundException' in str(e):
+            return []
+        raise
+
     rs = []
     for event in events:
         raw_message = event['message']
-        if 'START' in raw_message or 'END' in raw_message or 'REPORT' in raw_message:
+        if not raw_message or 'START' in raw_message or 'END' in raw_message or 'REPORT' in raw_message:
             continue
 
-        rs.append(json.loads(raw_message))
+        try:
+            rs.append(json.loads(raw_message))
+        except Exception:
+            rs.append(raw_message)
 
     return rs
